/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * zproxy - the cooperating compressing/decompressing proxy
 **************************************************************/

// exclude multiple inheritance of this file
#ifndef __PROXY_H
#define __PROXY_H

// need some types defined here
#include <netinet/in.h>

// need the cache type
#include "../zproxy/cache.h"

// proxy server constants
#define	DEFAULT_PORT	64203
#define DEFAULT_WORKERS	15

#define DEFAULT_ZHOST   "localhost"
#define DEFAULT_ZPORT   64202

#define RPCLIST_FILE    "zrpc_list"

// proxy version string
#define PROXY_VERSION	"1.0.0"

// define which proxy replacement algorithm to use
// comment out the next line to use LFU instead
#define USE_LRU_THRESHOLD

// HTTP Date Format for strftime()
#define HTTP_DATE_FMT	"%a, %d %b %Y %H:%M:%S GMT"

// don't duplicate the status enumeration
#ifndef _STATUS
#define _STATUS
// function return values
enum STATUS {ST_FAIL, ST_SUCCEED};
#endif

// server data structure
typedef struct {
    uint16_t port; // the server port   int
    int workers;   // worker thread count

    in_addr_t zproxy_addr; // the other proxy server's address
    uint16_t zproxy_port;  // the other proxy server's port
} server_data_t;

// the global cache and cache mutex
extern cache_t *cache;
extern pthread_mutex_t cache_mutex;
extern server_data_t server_data;

// function prototypes

// proxy.c
void print_version_info(void);
void print_help_info(void);
void parse_command_line(const int, const char **, server_data_t *);
short int start_server(const uint16_t, int *);
void wait_for_connections(int);
void *run_server(void *);
void respond_to_client(int);
void *run_worker(void *);
in_addr_t get_ip_address(const char *);
short int connect_to_server(const in_addr_t, const uint16_t, int *);
short int send_request(int, int, const char *, char *, const int, size_t *);
void stop_server(int);
short int read_rpc_list(void);
int main(int, char **);

// parse.c
void send_http_header(int, int, const char *, unsigned long int, const char *, time_t);
void send_error_page(int, int, const char *, const char *);
int get_next_line(const char *, char *, int);
void parse(int, const char *, size_t *, size_t *);

// pcpra.c
short int lfu(cache_t *, cachefile_t *);
short int lru_threshold(cache_t *, cachefile_t *, const unsigned int);
short int cache_try_replace(cache_t *, cachefile_t *);

#endif

/******************************************************************************
 *  zproxy v 1.0.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/
