/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * zproxy - the cooperating compressing/decompressing proxy
 **************************************************************/

#ifndef _CACHE_H
#define _CACHE_H

#include <stdio.h>
#include <sys/time.h>

#include "list.h"

#define DEFAULT_CACHE_SIZE 1048576 // 1 MB cache
#define DEFAULT_THRESHOLD  10 * 1024

typedef struct {
    unsigned long int size, limit;
    vlist_t *cache;
} cache_t;

typedef struct {
    char uri[2054];
    time_t last_access;
    unsigned long int use_count;
    char *file_map;
    size_t file_size;
} cachefile_t;

/* Function Prototypes */

// create a cache with an upper limit
cache_t *cache_create(unsigned long int);

// destroy a cache structure
void cache_destroy(cache_t *);

// add a cache file to the cache
// provide the cache, the URI, the last access time, the frequency of use count,
// the memory mapped file address, and the file size
// it will create the cachefile_t structure for storage and deallocation internally
short int cache_add(cache_t *, cachefile_t *);

// retrieve a cachefile from the cache using the URI
cachefile_t *cache_retrieve(cache_t *, const char *);

// retrieve a cachefile based on its location in the cache, indexed starting at 0
cachefile_t *cache_get(cache_t *, long int);

// remove a cachefile from the cache
void cache_remove(cache_t *, cachefile_t *);

#endif

/******************************************************************************
 *  zproxy v 1.0.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/
