/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * zproxy - the cooperating compressing/decompressing proxy
 **************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "list.h"

vlist_t *list_create(void) {
    vlist_t *list = NULL;

    if ((list = malloc(sizeof(vlist_t))) == NULL) {
	return NULL;
    }

    list->node = NULL;
    list->size = 0;
    list->index = -1;

    return list;
}

void list_destroy(vlist_t *list) {
    struct vlistnode_t *next = NULL;
    struct vlistnode_t *temp = NULL;

    do {
	temp = list->node;

	if (temp != NULL) {
	    next = temp->next;

	    free(temp);
	}
    } while (next != NULL);

    free(list);

    list = NULL;
}

void move_to_node(vlist_t *list, int index) {
    while (list->index < index) {
	list->index++;

	if (list->index < list->size) {
	    list->node = list->node->next;
	}
    }

    while (list->index > index) {
	list->index--;

	if (list->node->last != NULL) {
	    list->node = list->node->last;
	}
    }
}

short int list_insert(vlist_t *list, int index, void *element) {
    struct vlistnode_t *node = NULL;

    if (index > list->size) {
	index = list->size;
    }

    if (index < 0) {
	index = 0;
    }

    if ((node = malloc(sizeof(struct vlistnode_t))) == NULL) {
	return ST_FAIL;
    }

    node->element = element;
    node->next = NULL;
    node->last = NULL;

    if (list->size == 0) {
	list->index++;
	list->node = node;
    } else {
	move_to_node(list, index);

	if (list->index == list->size) {
	    node->last = list->node;
	    list->node->next = node;
	    list->node = list->node->next;
	} else if (list->index == 0) {
	    node->next = list->node;
	    list->node->last = node;
	    list->node = list->node->last;
	} else if (list->index < list->size) {
	    node->last = list->node->last;
	    node->next = list->node;
	    list->node->last->next = node;
	    list->node->last = node;
	    list->node = list->node->last;
	}
    }

    list->size++;

    return ST_SUCCEED;
}

short int list_append(vlist_t *list, void *element) {
    return list_insert(list, list->size, element);
}

void *list_get(vlist_t *list, int index) {
    if ((index >= list->size) || (index < 0)) {
	return NULL;
    }

    move_to_node(list, index);

    return list->node->element;
}

void *list_find(vlist_t *list, void *element) {
    return list_get(list, list_index_of(list, element));
}

void *list_remove(vlist_t *list, int index) {
    struct vlistnode_t *temp;
    void *element = NULL;

    if ((index >= list->size) || (index < 0)) {
	return NULL;
    }

    move_to_node(list, index);
    element = list->node->element;

    if (list->index == 0) {
	if (list->size == 1) {
	    free(list->node);
	    list->node = NULL;
	} else {
	    list->node->next->last = NULL;
	    temp = list->node;
	    list->node = list->node->next;
	    free(temp);
	}
    } else if (list->index == (list->size - 1)) {
	list->node->last->next = NULL;
	temp = list->node;
	list->node = list->node->last;
	list->index--;
	free(temp);
    } else {
	list->node->last->next = list->node->next;
	list->node->next->last = list->node->last;
	temp = list->node;
	list->node = list->node->next;
	free(temp);
    }

    list->size--;

    return element;
}

int list_index_of(vlist_t *list, void *element) {
    move_to_node(list, 0);

    while (list->index < list->size) {
	if (list->node->element == element) {
	    return list->index;
	}

	list->index++;

	if (list->node->next != NULL) {
	    list->node = list->node->next;
	}
    }

    return -1;
}

/******************************************************************************
 *  zproxy v 1.0.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/
