/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * zproxy - the cooperating compressing/decompressing proxy
 **************************************************************/

#ifndef _LIST_H
#define _LIST_H

#ifndef _STATUS
#define _STATUS
enum STATUS {ST_FAIL, ST_SUCCEED};
#endif

typedef struct {
    long int size, index;
    struct vlistnode_t *node;
} vlist_t;

struct vlistnode_t {
    struct vlistnode_t *next, *last;
    void *element;
};

/* function prototypes */

// create the linked list
// returns list structure if successful, NULL otherwise
vlist_t *list_create(void);

// destroy the linked list
void list_destroy(vlist_t *);

// insert element into list in position n
short int list_insert(vlist_t *, int, void *);

// insert element at the end of the list
short int list_append(vlist_t *, void *);

// get an element in the list at index n
void *list_get(vlist_t *, int);

// find the element in the list, or return NULL if it's not in the list
void *list_find(vlist_t *, void *);

// remove an element from the list at index n
void *list_remove(vlist_t *, int);

// find the index of an element in the list, or -1 if it's not in the list
int list_index_of(vlist_t *, void *);

#endif

/******************************************************************************
 *  zproxy v 1.0.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/
