/*************************************************************
 * vqueue Queue library
 * Version 1.0.0
 *
 * Implementation of a FIFO queue in C
 *
 * Copyright (C) 2002
 * John David Ratliff [jdratlif@cs.indiana.edu]
 * Chris O'Donnell    [chodonne@cs.indiana.edu]
 **************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "queue.h"

// create a new queue of size 'size'
vqueue_t *queue_create(unsigned long int size) {
    vqueue_t *queue = NULL;
	
    // allocate memory for the queue structure
    if ((queue = (vqueue_t *)malloc(sizeof(vqueue_t))) == NULL) {
	fprintf(stderr,"insufficient memory to allocate queue structure.\n");
	return NULL;
    }
	
    // setup the queue data
    queue->size = size;
    queue->in = 0;
    queue->out = 0;
    queue->length = 0;

    // allocate the queue data array
    if ((queue->data = (void *)malloc(size * sizeof(void *))) == NULL) {
	fprintf(stderr,
		"insufficient memory to allocate queue of size %lu.\n", size);
		
	// free the queue structure
	free(queue);
	return NULL;
    }
	
    // return the newly allocated queue
    return queue;
}

// destroy the queue structure
void queue_destroy(vqueue_t *queue) {
    // free the allocated queue memory
    if (queue != NULL) {
	free(queue->data);
	free(queue);
    }
	
    // assign queue to be NULL
    queue = NULL;
}

// check if the queue is empty
short int queue_empty(vqueue_t *queue) {
    if (queue != NULL) {
	if (queue->length == 0) {
	    return TRUE;
	}
    }
	
    return FALSE;
}

// checks if the queue is full
short int queue_full(vqueue_t *queue) {
    if (queue != NULL) {
	if (queue->length == queue->size) {
	    return TRUE;
	}
    }
	
    return FALSE;
}

// enqueue the item 'item' onto the queue
short int queue_enqueue(vqueue_t *queue, void *item) {
    if (queue != NULL) {
	if (queue_full(queue)) {
	    return FALSE;
	}
		
	// enqueue the item and update the queue
	queue->data[queue->in] = item;
	queue->in++;
	queue->length++;
		
	if (queue->in == queue->size) {
	    queue->in = 0;
	}
    } else {
	return FALSE;
    }
	
    return TRUE;
}

// dequeue an item from the queue into 'item'
short int queue_dequeue(vqueue_t *queue, void **item) {
    if (queue != NULL) {
	if (queue_empty(queue)) {
	    return FALSE;
	}
		
	// dequeue the item and update the queue
	*item = queue->data[queue->out];
	queue->out++;
	queue->length--;
		
	if (queue->out == queue->size) {
	    queue->out = 0;
	}
    } else {
	return FALSE;
    }
	
    return TRUE;
}

/******************************************************************************
 *  vqueue Queue Library
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

