/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * zproxy - the cooperating compressing/decompressing proxy
 **************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "../zlib/zlib.h"
#include "zip.h"

#define BUFFER_SIZE 1024*1024

//zipdata_t *zip_1_svc(zipdata_t *znow, struct svc_req *rqstp) {
bool_t zip_1_svc(zipdata_t *znow, zipdata_t *zreturn, struct svc_req *rqstp) {
    unsigned long int size = BUFFER_SIZE;

    // zero out our structure
    memset(zreturn, 0, sizeof(zipdata_t));

    // allocate memory for the compression
    if ((zreturn->data.data_val = (char *)malloc(BUFFER_SIZE)) == NULL) {
	fprintf(stderr,"fatal error mem zip_1\n");

	// no more free memory -- fatal error
	return FALSE;
    }

    // compress the buffer using best compression
    compress2(zreturn->data.data_val, &size, znow->data.data_val, znow->data.data_len, Z_BEST_COMPRESSION);

    // set the size of the compression buffer...
    zreturn->data.data_len = size;

    return TRUE;
}

//zipdata_t *unzip_1_svc(zipdata_t *znow, struct svc_req *rqstp) {
bool_t unzip_1_svc(zipdata_t *znow, zipdata_t *zreturn, struct svc_req *rqstp) {
    unsigned long int size = BUFFER_SIZE;

    // zero out our structure
    memset(zreturn, 0, sizeof(zipdata_t));

    // allocate memory for the compression
    if ((zreturn->data.data_val = (char *)malloc(BUFFER_SIZE)) == NULL) {
	fprintf(stderr,"fatal error mem unzip_1\n");

	// no more free memory -- fatal error
	return FALSE;
    }

    // uncompress the data
    uncompress(zreturn->data.data_val, &size, znow->data.data_val, znow->data.data_len);

    // set the size of the compression buffer...
    zreturn->data.data_len = size;

    return TRUE;
}

int zip_prog_1_freeresult (SVCXPRT *transp, xdrproc_t xdr_result, caddr_t result) {
    xdr_free(xdr_result, result);

    return 1;
}

/******************************************************************************
 *  zproxy v 1.0.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/
